#include "mbed.h"
#include "SimpleModbus.h"

static Ticker ticker;
static comfileHMI HMI;

void StartModbus(PinName tx, PinName rx, long baud,u8 addr ,u16 *Register, u8 *Coil )
{
    //comfileHMI HMI;
    static RawSerial Device(tx, rx); 
    Device.baud(baud);
    Device.format(8, SerialBase::None, 1);
    HMI.init_comfileHMI(&Device, addr, Register, Coil);
    Device.attach(&HMI, &comfileHMI::Rx_interrupt, Serial::RxIrq);    
}

void comfileHMI::init_comfileHMI(RawSerial* Serial, u8 addr, u16 *Register, u8 *Coil)
{
         __modbusComCh=0;
         __modbusSlaveAdr=0;
         __modbusBroadMode = 0;
         __modbusFoundFrame = 0;
         __modbusIntervalCounter=0;;
        
         __modbusWordBuffer = NULL;
         __modbusBitBuffer = NULL;
        
         __save_pointer[0]=0;
         __save_pointer[1]=0;
         __save_pointer[2]=0;
         __save_pointer[3]=0;
         
         __action_pointer[0]=__ring_buffer_max;
         __action_pointer[1]=__ring_buffer_max;
         __action_pointer[2]=__ring_buffer_max;
         __action_pointer[3]=__ring_buffer_max;
         
         __modbusSwitch[0]=0;
         __modbusSwitch[1]=0;
         __modbusSwitch[2]=0;
         __modbusSwitch[3]=0;
         __modbusBlankCheck = 0;
         
         device = Serial;
         startSimpleModbus(0, addr, Register, Coil);
         ticker.attach_us(this, &comfileHMI::RunTime_HMI, 500);
}

/*******************************************************************************
* Function Name  : SysTickHandler (1mS마다 걸린다.)
*******************************************************************************/
// 이 숫자를 바꾸면, 모드버스 수신후 응답 간격이 바뀐다. 5=5mS, 
#define __modbusReturnInterval 5
// 1mS 마다 인터럽트가 걸리는 루틴
// 이곳에서 모드버스관련 처리를 한다.

//Ticker timer_1;
void comfileHMI::Rx_interrupt()
{
    while (device->readable()){ push_ringbuffer( 0,device->getc() );}
    return;
}

void comfileHMI::RunTime_HMI()
{
  //  MODBUS관련 처리 
  if (__modbusFoundFrame) {
    __modbusIntervalCounter++;
    if (__modbusIntervalCounter > __modbusReturnInterval) {
      __modbusIntervalCounter = 0;
      __modbusFoundFrame = 0;
      modbusMainProcessing();
    }
   }
//  프레임 사이 블랭크 구간체크를 위한 카운터 업 
  if (__modbusBlankCheck < 0xffff) __modbusBlankCheck++;
}

void comfileHMI::push_ringbuffer(u8 comch, u8 comdt)
{
  if (__modbusSwitch[comch]) {
    if (__modbusBlankCheck > 500) {
        comFlush(comch); // ^^ 500mS쉬었다 들어온 경우, 기존버퍼에 있던거는 모두 지운다. (노이즈 데이터일 가능성 있음)
        }
    __modbusBlankCheck = 0;
    }

  if(__save_pointer[comch]!=__action_pointer[comch]){  // 버퍼가 꽉차면 더이상 넣치 않는다. 
    __ring_buffer[comch][__save_pointer[comch]]=comdt;
    __save_pointer[comch]++;
    if(__save_pointer[comch]>__ring_buffer_max) { 
      __save_pointer[comch]=0; 
      }
  }  
    if (__modbusSwitch[comch]) modbusFrameSearch(comch); //수신직후 해당 채널이 모드버스 활성화 되어있으면 처리해준다.
}

u8 comfileHMI::read_ringbuffer(u8 comch)
{
  __action_pointer[comch]++;
  if(__action_pointer[comch] > __ring_buffer_max){ 
    __action_pointer[comch]=0; 
    }  
  return(__ring_buffer[comch][__action_pointer[comch]]);
}

/*******************************************************************************
  Function Name  : comFlush   [ 링버퍼 비우기 ]
*******************************************************************************/
void comfileHMI::comFlush(u8 comch)
{ 
  __action_pointer[comch] = __ring_buffer_max;
  __save_pointer[comch] = 0;
}
/*******************************************************************************
  Function Name  : comLen  [  ring buffer에 있는 데이터 량을 리턴  ]
*******************************************************************************/
u16 comfileHMI::comLen(u8 comch)
{ 
  u16 i;  
  if (__save_pointer[comch]>(__action_pointer[comch]+1)) { 
    i=__save_pointer[comch]-__action_pointer[comch]-1;
    }
  else if (__save_pointer[comch]<(__action_pointer[comch]+1)) {
    i=__ring_buffer_max-__action_pointer[comch]+__save_pointer[comch]; 
    }
  else {
    i=0;
    }
  return(i);
}
/*******************************************************************************
  Function Name  : comGet  [ 1바이트 읽기 ]
*******************************************************************************/
short comfileHMI::comGet(u8 comch)
{
  if (comLen(comch) == 0) return -1;
  return read_ringbuffer(comch);
}
/*******************************************************************************
  Function Name  : comGets  [ n바이트 읽기 ]
*******************************************************************************/
short comfileHMI::comGets(u8 comch, u8 * dest, u16 length)
{
  u16 i;
  if (comLen(comch) == 0) return 0;
  for (i=0; i<length; i++) {
    dest[i] = read_ringbuffer(comch);
    }
  return -1;
}

/*******************************************************************************
  Function Name  : comPut [ 1바이트 쓰기]
*******************************************************************************/


void comfileHMI::comPut(u8 comch, u8 cha)
{
    device->putc(cha);
    while(!device->writeable());
}

/*******************************************************************************
  Function Name  : comPuts [ n바이트 쓰기 ]
*******************************************************************************/
void comfileHMI::comPuts(u8 comch, u8 * dest, u16 length)
{
  u16 i;
  for (i=0; i<length; i++) {
    comPut(comch,dest[i]);
    }
}

u8 comfileHMI::mb_get_byte_rtu()
{
  return read_ringbuffer(__modbusComCh);
}

u16 comfileHMI::mb_get_word_rtu()
{
  u16 wi;
  wi = read_ringbuffer(__modbusComCh);
  wi <<= 8;
  wi += read_ringbuffer(__modbusComCh);
  return wi;
}

void comfileHMI::mb_inc_pointer()
{
  (void)read_ringbuffer(__modbusComCh);
}

void comfileHMI::mb_put_byte_rtu(u8 dt)
{
  comPut(__modbusComCh, dt);
  mb_crc_compute(dt);
}

//
//  Bit 읽기 RTU (FC 0,1)
//
const u8 mb_ncdtb[] = {1,2,4,8,16,32,64,128};

u8 comfileHMI::mb_collectbits(u16 adr)
{
  u8 i,j,m=0;               // adr은 modbus상의 주소, 이것을 실제주소로 환산하여 
  for (i=0;i<8;i++) {       // 그곳에 있는 비트값을 8번 읽어, 한바이트의 데이터로 완성시켜준다.
    j = __modbusBitBuffer[adr>>3];
    if (j & mb_ncdtb[adr & 7]) m |= mb_ncdtb[i];
    adr++;
    }
  return m;
}

void comfileHMI::mbcmd_read_bit_rtu(u8 cmd)
{
  u8 i,j,bln;
  u16 stadr,ln;
  stadr = mb_get_word_rtu(); // Start Address
  ln = mb_get_word_rtu(); // Length
  mb_inc_pointer();
  mb_inc_pointer(); // CRC는 이미 체크했으므로 pass
  // 송신시작
  if (__modbusBroadMode) return;    // Brodecast 모드면 그냥 리턴 
  uchCRCHi = uchCRCLo = 0xff;   // CRC 계산 start
  mb_put_byte_rtu(__modbusSlaveAdr);       // Slave Address
  mb_put_byte_rtu(cmd);         // Command
  bln = ln >> 3;
  if (ln & 7) bln++;          // 나머지가 있으면 +1
  mb_put_byte_rtu(bln);           // Byte Count
  for (i=0; i<bln; i++) {
    j=mb_collectbits(stadr);
    mb_put_byte_rtu(j);
    stadr += 8;
    }
  i = uchCRCLo; // 바로 아래에서 uchCRCLo를 건드리기 때문에 Save해둔다.
  mb_put_byte_rtu(uchCRCHi); // CRC
  mb_put_byte_rtu(i);
}

//
//  1 Bit 쓰기 (FC 5)
//

void comfileHMI::mbcmd_write_bit_single_rtu()
{
  u8 i;
  u16 stadr,realadr,val;
  stadr = mb_get_word_rtu(); // Start Address
  val = mb_get_word_rtu(); // Write Value
  mb_inc_pointer(); // CRC는 이미 체크했으므로 pass
  mb_inc_pointer();
  
  realadr = stadr >> 3;
  if (val) __modbusBitBuffer[realadr] |= mb_ncdtb[stadr & 7];
  else __modbusBitBuffer[realadr] &= ~mb_ncdtb[stadr & 7];
  // 송신시작
  if (__modbusBroadMode) return;    // Brodecast 모드면 그냥 리턴 
  uchCRCHi = uchCRCLo = 0xff;   // CRC 계산 start
  mb_put_byte_rtu(__modbusSlaveAdr);       // Slave Address
  mb_put_byte_rtu(5);           // Command
  mb_put_byte_rtu(stadr >> 8);
  mb_put_byte_rtu(stadr);
  mb_put_byte_rtu(val >> 8);
  mb_put_byte_rtu(val);
  i = uchCRCLo;
  mb_put_byte_rtu(uchCRCHi); // CRC
  mb_put_byte_rtu(i);
}

//
//  MODBUS RTU Word Read  (FC 3, 4)
//

void comfileHMI::mbcmd_read_word_rtu(u8 cmd)
{
  u8 i;
  u16 stadr,ln,li,dt;
  stadr = mb_get_word_rtu(); // Start Address
  ln = mb_get_word_rtu(); // Length
  mb_inc_pointer();
  mb_inc_pointer();   // CRC는 이미 체크되었으므로 pass
  //
  //  Send Start
  //
  if (__modbusBroadMode) return;    // Brodecast 모드면 그냥 리턴 
  uchCRCHi = uchCRCLo = 0xff;   // CRC 계산 start
  mb_put_byte_rtu(__modbusSlaveAdr);       // Slave Address
  mb_put_byte_rtu(cmd);         // Command
  i = ln << 1;
  mb_put_byte_rtu(i);           // Byte Count

  for (li=0; li<ln; li++) {
    dt = __modbusWordBuffer[stadr++];
    mb_put_byte_rtu(dt >> 8); // High    
    mb_put_byte_rtu(dt); // Low
    }
  i = uchCRCLo;
  mb_put_byte_rtu(uchCRCHi); // CRC
  mb_put_byte_rtu(i);
}

//
//    1워드만 쓰기 (FC 6)
//

void comfileHMI::mbcmd_write_word_single_rtu()
{
  //u8 lsave,i;
  //u16 stadr,realadr,val;
  u8 i;
  u16 stadr,val;
  stadr = mb_get_word_rtu(); // Start Address
  val = mb_get_word_rtu(); // Write Value
  mb_inc_pointer();
  mb_inc_pointer();   // CRC는 이미 체크되었으므로 pass
  __modbusWordBuffer[stadr] = val;
    
  if (__modbusBroadMode) return;    // Brodecast 모드면 그냥 리턴 
  uchCRCHi = uchCRCLo = 0xff;   // CRC 계산 start
  mb_put_byte_rtu(__modbusSlaveAdr);       // Slave Address
  mb_put_byte_rtu(6);           // Command
  mb_put_byte_rtu(stadr >> 8);
  mb_put_byte_rtu(stadr);
  mb_put_byte_rtu(val >> 8);
  mb_put_byte_rtu(val);
  i = uchCRCLo; // 바로 아래에서 uchCRCLo를 건드리기 때문에 Save해둔다.
  mb_put_byte_rtu(uchCRCHi); // CRC
  mb_put_byte_rtu(i);
}


//
//  연속으로 Bit 쓰기 RTU (FC 15)
//

void comfileHMI::mb_writebits(u16 adr, u8 val, u8 ln)
{
  u8 i,m;
  if (ln > 7) m=8;
    else m=ln;   // 15인 경우도 8개씩 나누어서 처리 
  for (i=0;i<m;i++) { // 8번 (마지막에서는 8번 미만) 돌면서, buffer의 내용을 수정한다.
    if (val & mb_ncdtb[i]) // (adr에 byte어드레스와 bit어드레스 모두 포함되어 있다.)
      __modbusBitBuffer[adr >> 3] |= mb_ncdtb[adr & 7];
    else 
      __modbusBitBuffer[adr >> 3] &= ~mb_ncdtb[adr & 7];
    adr++;
    }
}

void comfileHMI::mbcmd_write_bit_multiple_rtu()
{
  u8 i,j,cnt;
  u16 stadr,ln,lnsave,stadr_save;
  stadr_save = stadr = mb_get_word_rtu(); // Start Address
  lnsave = ln = mb_get_word_rtu(); // Length
  cnt = mb_get_byte_rtu(); // byte count
  for (i=0; i<cnt; i++) {
    j = mb_get_byte_rtu(); // value
    mb_writebits(stadr, j, ln);
    stadr += 8;
    ln -= 8;
    }
  mb_inc_pointer();
  mb_inc_pointer();     // CRC는 이미 체크했으므로 pass
  // 송신시작
  if (__modbusBroadMode) return;    // Brodecast 모드면 그냥 리턴 
  uchCRCHi = uchCRCLo = 0xff;   // CRC 계산 start
  mb_put_byte_rtu(__modbusSlaveAdr);       // Slave Address
  mb_put_byte_rtu(15);         // Command
  mb_put_byte_rtu(stadr_save >> 8);
  mb_put_byte_rtu(stadr_save);
  mb_put_byte_rtu(lnsave >> 8);
  mb_put_byte_rtu(lnsave);
  i = uchCRCLo; // 바로 아래에서 uchCRCLo를 건드리기 때문에 Save해둔다.
  mb_put_byte_rtu(uchCRCHi); // CRC
  mb_put_byte_rtu(i);
}

//
//    연속으로 워드 쓰기 RTU (FC 16)
//

void comfileHMI::mbcmd_write_word_multiple_rtu()
{
  u8 j;
  u16 stadr,realadr,val,ln,i;
  realadr = stadr = mb_get_word_rtu(); // Start Address
  ln = mb_get_word_rtu(); // length
  mb_inc_pointer(); // byte count
  for (i=0;i<ln;i++) {
    val = mb_get_word_rtu(); // Write Value
    __modbusWordBuffer[realadr++] = val;      // 여기서 Write 실행
    }
  mb_inc_pointer();
  mb_inc_pointer(); // CRC는 이미 체크했으므로 pass
  // 송신시작
  if (__modbusBroadMode) return;    // Brodecast 모드면 그냥 리턴 
  uchCRCHi = uchCRCLo = 0xff;   // CRC 계산 start
  mb_put_byte_rtu(__modbusSlaveAdr);       // Slave Address
  mb_put_byte_rtu(16);           // Command
  mb_put_byte_rtu(stadr >> 8);
  mb_put_byte_rtu(stadr);
  mb_put_byte_rtu(ln >> 8);
  mb_put_byte_rtu(ln);
  j = uchCRCLo; // 바로 아래에서 uchCRCLo를 건드리기 때문에 Save해둔다.
  mb_put_byte_rtu(uchCRCHi); // CRC
  mb_put_byte_rtu(j);
}


/*******************************************************************************

  Function Name  : startModbus
  모드버스 MAIN 처리
  
*******************************************************************************/

const u8 auchCRCH[]  =  {
0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,
0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01,
0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41,
0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81,
0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0,
0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01,
0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40,
0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,
0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01,
0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,
0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0,
0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01,
0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81, 0x40, 0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41,
0x00, 0xC1, 0x81, 0x40, 0x01, 0xC0, 0x80, 0x41, 0x01, 0xC0, 0x80, 0x41, 0x00, 0xC1, 0x81,0x40};

const u8 auchCRCL[] = {
0x00, 0xC0, 0xC1, 0x01, 0xC3, 0x03, 0x02, 0xC2, 0xC6, 0x06, 0x07, 0xC7, 0x05, 0xC5, 0xC4,
0x04, 0xCC, 0x0C, 0x0D, 0xCD, 0x0F, 0xCF, 0xCE, 0x0E, 0x0A, 0xCA, 0xCB, 0x0B, 0xC9, 0x09,
0x08, 0xC8, 0xD8, 0x18, 0x19, 0xD9, 0x1B, 0xDB, 0xDA, 0x1A, 0x1E, 0xDE, 0xDF, 0x1F, 0xDD,
0x1D, 0x1C, 0xDC, 0x14, 0xD4, 0xD5, 0x15, 0xD7, 0x17, 0x16, 0xD6, 0xD2, 0x12, 0x13, 0xD3,
0x11, 0xD1, 0xD0, 0x10, 0xF0, 0x30, 0x31, 0xF1, 0x33, 0xF3, 0xF2, 0x32, 0x36, 0xF6, 0xF7,
0x37, 0xF5, 0x35, 0x34, 0xF4, 0x3C, 0xFC, 0xFD, 0x3D, 0xFF, 0x3F, 0x3E, 0xFE, 0xFA, 0x3A,
0x3B, 0xFB, 0x39, 0xF9, 0xF8, 0x38, 0x28, 0xE8, 0xE9, 0x29, 0xEB, 0x2B, 0x2A, 0xEA, 0xEE,
0x2E, 0x2F, 0xEF, 0x2D, 0xED, 0xEC, 0x2C, 0xE4, 0x24, 0x25, 0xE5, 0x27, 0xE7, 0xE6, 0x26,
0x22, 0xE2, 0xE3, 0x23, 0xE1, 0x21, 0x20, 0xE0, 0xA0, 0x60, 0x61, 0xA1, 0x63, 0xA3, 0xA2,
0x62, 0x66, 0xA6, 0xA7, 0x67, 0xA5, 0x65, 0x64, 0xA4, 0x6C, 0xAC, 0xAD, 0x6D, 0xAF, 0x6F,
0x6E, 0xAE, 0xAA, 0x6A, 0x6B, 0xAB, 0x69, 0xA9, 0xA8, 0x68, 0x78, 0xB8, 0xB9, 0x79, 0xBB,
0x7B, 0x7A, 0xBA, 0xBE, 0x7E, 0x7F, 0xBF, 0x7D, 0xBD, 0xBC, 0x7C, 0xB4, 0x74, 0x75, 0xB5,
0x77, 0xB7, 0xB6, 0x76, 0x72, 0xB2, 0xB3, 0x73, 0xB1, 0x71, 0x70, 0xB0, 0x50, 0x90, 0x91,
0x51, 0x93, 0x53, 0x52, 0x92, 0x96, 0x56, 0x57, 0x97, 0x55, 0x95, 0x94, 0x54, 0x9C, 0x5C,
0x5D, 0x9D, 0x5F, 0x9F, 0x9E, 0x5E, 0x5A, 0x9A, 0x9B, 0x5B, 0x99, 0x59, 0x58, 0x98, 0x88,
0x48, 0x49, 0x89, 0x4B, 0x8B, 0x8A, 0x4A, 0x4E, 0x8E, 0x8F, 0x4F, 0x8D, 0x4D, 0x4C, 0x8C,
0x44, 0x84, 0x85, 0x45, 0x87, 0x47, 0x46, 0x86, 0x82, 0x42, 0x43, 0x83, 0x41, 0x81, 0x80,0x40};

void comfileHMI::mb_crc_compute(u8 rch)
{
  uINDEX = uchCRCHi ^ rch;
  uchCRCHi = uchCRCLo ^ auchCRCH[uINDEX];
  uchCRCLo = auchCRCL[uINDEX];
}

//
//  CRC16을 얻기위한 함수, byte형 어레이에서 원하는 바이트수만큼 계산한뒤 그 결과값을 리턴
//

u16 comfileHMI::getCrc(u8 * targetArray, u16 datalength)
{
  u16 d;
  uchCRCHi = uchCRCLo = 0xff;   // CRC 계산 start
  for (d=0; d<datalength; d++) {
    mb_crc_compute(targetArray[d]);
    }
  d = (uchCRCHi << 8) + uchCRCLo;
  return d;
}

//  Register (워드저장소)의 시작번지와
//  Coil (비트저장소)의 시작번지를 건네 받는다. 
//  응답 간격은 대략 3mS

void comfileHMI::startSimpleModbus(u8 comch, u8 modslaveadr, u16 * modbusBufferRegister, u8 * modbusBufferCoil)
{
  __modbusComCh = comch;
  __modbusSlaveAdr = modslaveadr;
  __modbusSwitch[0] = 0;
  __modbusSwitch[1] = 0;
  __modbusSwitch[2] = 0;
  __modbusSwitch[3] = 0; 
  __modbusSwitch[comch] = 1; // On, 4개 채널중 한개만 사용가능
  __modbusWordBuffer = modbusBufferRegister;
  __modbusBitBuffer = modbusBufferCoil;
}

//
// 1바이트 수신이 되었을때, 모드버스 RTU 프레임을 완성 (끝에 CRC가 앞에거와 일치)하는지 여부를 조사한다.
//

void comfileHMI::modbusFrameSearch(u8 comch)
{
  u8 uch, ucl, bh, bl;
  u16 mclen, i;
  u16 uix;
  u16 save_pointer;
  mclen = comLen(comch); // 수신된 바이트 수 

  if (mclen < 8) return; // 8바이트 보다 적은 코멘드는 없으므로 바로 리턴

  uch = ucl = 0xff;       // 이제 CRC를 계산하기 시작..  이것을 위해 링버퍼를 스캔한다. 
  save_pointer = __action_pointer[comch]; // 실행지점 포인터를 카피 
  for (i = 0; i< mclen-2; i++) {
    uix = uch ^ mb_get_byte_rtu(); // __action_pointer를 건드리므로 나중에 복구해주어야 한다.
    uch = ucl ^ auchCRCH[uix];
    ucl = auchCRCL[uix];
    }
  bh = mb_get_byte_rtu(); //이것이 CRC값 
  bl = mb_get_byte_rtu();
  __action_pointer[comch] = save_pointer; // 복구 

  if ((bh == uch) && (bl == ucl)) __modbusFoundFrame = 1; // 발견!
    
}

void comfileHMI::modbusMainProcessing()
{
  u8 fc,sladr;
  sladr = mb_get_byte_rtu(); // 슬레이브 어드레스 읽어오기 
  if (sladr == 0) {
    __modbusBroadMode = 1; // 슬레이브 어드레스가 0이면 브로드케스트 모드 = 1
    }
  else {
    if (sladr != __modbusSlaveAdr) { // 슬레이브 어드레스가 일치하지 않으면, 버퍼 클리어후 리턴 
      comFlush(__modbusComCh);
      return;
      }
   }
 
  fc = mb_get_byte_rtu(); // 펑션코드 읽어오기 
  
  //comPut(0, sladr);
  //comPut(0, fc);
 
  switch (fc) {
  case 1:
  case 2:
    mbcmd_read_bit_rtu(fc);
    break;
  case 3:
  case 4:
    mbcmd_read_word_rtu(fc);
    break;
  case 5:
    mbcmd_write_bit_single_rtu();
    break;
  case 6:
    mbcmd_write_word_single_rtu();
    break;
  case 15:
    mbcmd_write_bit_multiple_rtu();
    break;
  case 16:
    mbcmd_write_word_multiple_rtu();
    break;
  default:
    comFlush(__modbusComCh); // 없는 FC(펑션코드)일경우, 버퍼클리어 
    return;
    }
}  



